@@ -27,7 +27,7 @@ where
     Staking(StakingMsg),
     #[cfg(feature = "staking")]
     Distribution(DistributionMsg),
-    /// A Stargate message encoded the same way as a protobof [Any](https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/any.proto).
+    /// A Stargate message encoded the same way as a protobuf [Any](https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/any.proto).
     /// This is the same structure as messages in `TxBody` from [ADR-020](https://github.com/cosmos/cosmos-sdk/blob/master/docs/architecture/adr-020-protobuf-transaction-encoding.md)
     #[cfg(feature = "stargate")]
     Stargate {
@@ -37,6 +37,8 @@ where
     #[cfg(feature = "stargate")]
     Ibc(IbcMsg),
     Wasm(WasmMsg),
+    #[cfg(feature = "stargate")]
+    Gov(GovMsg),
 }
 
 /// The message types of the bank module.
@@ -120,7 +122,7 @@ pub enum WasmMsg {
         contract_addr: String,
         /// msg is the json-encoded ExecuteMsg struct (as raw Binary)
         msg: Binary,
-        send: Vec<Coin>,
+        funds: Vec<Coin>,
     },
     /// Instantiates a new contracts from previously uploaded Wasm code.
     ///
@@ -131,7 +133,7 @@ pub enum WasmMsg {
         code_id: u64,
         /// msg is the JSON-encoded InstantiateMsg struct (as raw Binary)
         msg: Binary,
-        send: Vec<Coin>,
+        funds: Vec<Coin>,
         /// A human-readbale label for the contract
         label: String,
     },
@@ -160,39 +162,54 @@ pub enum WasmMsg {
     ClearAdmin { contract_addr: String },
 }
 
+#[cfg(feature = "stargate")]
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
+#[serde(rename_all = "snake_case")]
+pub enum GovMsg {
+    /// This maps directly to [MsgVote](https://github.com/cosmos/cosmos-sdk/blob/v0.42.5/proto/cosmos/gov/v1beta1/tx.proto#L46-L56) in the Cosmos SDK with voter set to the contract address.
+    Vote { proposal_id: u64, vote: VoteOption },
+}
+
+#[cfg(feature = "stargate")]
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
+#[serde(rename_all = "snake_case")]
+pub enum VoteOption {
+    Yes,
+    No,
+    Abstain,
+    NoWithVeto,
+}
+
 /// Shortcut helper as the construction of WasmMsg::Instantiate can be quite verbose in contract code.
 ///
 /// When using this, `admin` is always unset. If you need more flexibility, create the message directly.
-pub fn wasm_instantiate<T>(
+pub fn wasm_instantiate(
     code_id: u64,
-    msg: &T,
-    send: Vec<Coin>,
+    msg: &impl Serialize,
+    funds: Vec<Coin>,
     label: String,
-) -> StdResult<WasmMsg>
-where
-    T: Serialize,
-{
+) -> StdResult<WasmMsg> {
     let payload = to_binary(msg)?;
     Ok(WasmMsg::Instantiate {
         admin: None,
         code_id,
         msg: payload,
-        send,
+        funds,
         label,
     })
 }
 
 /// Shortcut helper as the construction of WasmMsg::Instantiate can be quite verbose in contract code
-pub fn wasm_execute<T, U>(contract_addr: T, msg: &U, send: Vec<Coin>) -> StdResult<WasmMsg>
-where
-    T: Into<String>,
-    U: Serialize,
-{
+pub fn wasm_execute(
+    contract_addr: impl Into<String>,
+    msg: &impl Serialize,
+    funds: Vec<Coin>,
+) -> StdResult<WasmMsg> {
     let payload = to_binary(msg)?;
     Ok(WasmMsg::Execute {
         contract_addr: contract_addr.into(),
         msg: payload,
-        send,
+        funds,
     })
 }
 
@@ -244,6 +261,16 @@ where
     }
 }
 
+#[cfg(feature = "stargate")]
+impl<T> From<GovMsg> for CosmosMsg<T>
+where
+    T: Clone + fmt::Debug + PartialEq + JsonSchema,
+{
+    fn from(msg: GovMsg) -> Self {
+        CosmosMsg::Gov(msg)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
